@@ -10,6 +10,8 @@ declare(strict_types=1);
 namespace Nette\Utils;
 
 use Nette;
+use function is_array, is_int, is_string;
+use const IMG_BMP, IMG_FLIP_BOTH, IMG_FLIP_HORIZONTAL, IMG_FLIP_VERTICAL, IMG_GIF, IMG_JPG, IMG_PNG, IMG_WEBP, PATHINFO_EXTENSION;
 
 
 /**
@@ -22,7 +24,7 @@ use Nette;
  * $image->send();
  * </code>
  *
- * @method Image affine(array $affine, ?array $clip = null)
+ * @method Image affine(array<int, float|int> $affine, ?array{x: int, y: int, width: int, height: int} $clip = null)
  * @method void alphaBlending(bool $enable)
  * @method void antialias(bool $enable)
  * @method void arc(int $centerX, int $centerY, int $width, int $height, int $startAngle, int $endAngle, ImageColor $color)
@@ -39,51 +41,51 @@ use Nette;
  * @method int colorResolve(int $red, int $green, int $blue)
  * @method int colorResolveAlpha(int $red, int $green, int $blue, int $alpha)
  * @method void colorSet(int $index, int $red, int $green, int $blue, int $alpha = 0)
- * @method array colorsForIndex(int $color)
+ * @method array{red: int, green: int, blue: int, alpha: int} colorsForIndex(int $color)
  * @method int colorsTotal()
  * @method int colorTransparent(?int $color = null)
- * @method void convolution(array $matrix, float $div, float $offset)
+ * @method void convolution(array<int, array<int, float>> $matrix, float $div, float $offset)
  * @method void copy(Image $src, int $dstX, int $dstY, int $srcX, int $srcY, int $srcW, int $srcH)
  * @method void copyMerge(Image $src, int $dstX, int $dstY, int $srcX, int $srcY, int $srcW, int $srcH, int $pct)
  * @method void copyMergeGray(Image $src, int $dstX, int $dstY, int $srcX, int $srcY, int $srcW, int $srcH, int $pct)
  * @method void copyResampled(Image $src, int $dstX, int $dstY, int $srcX, int $srcY, int $dstW, int $dstH, int $srcW, int $srcH)
  * @method void copyResized(Image $src, int $dstX, int $dstY, int $srcX, int $srcY, int $dstW, int $dstH, int $srcW, int $srcH)
- * @method Image cropAuto(int $mode = IMG_CROP_DEFAULT, float $threshold = .5, ?ImageColor $color = null)
+ * @method Image cropAuto(int $mode = 0, float $threshold = .5, ?ImageColor $color = null)
  * @method void ellipse(int $centerX, int $centerY, int $width, int $height, ImageColor $color)
  * @method void fill(int $x, int $y, ImageColor $color)
  * @method void filledArc(int $centerX, int $centerY, int $width, int $height, int $startAngle, int $endAngle, ImageColor $color, int $style)
  * @method void filledEllipse(int $centerX, int $centerY, int $width, int $height, ImageColor $color)
- * @method void filledPolygon(array $points, ImageColor $color)
+ * @method void filledPolygon(array<int, int> $points, ImageColor $color)
  * @method void filledRectangle(int $x1, int $y1, int $x2, int $y2, ImageColor $color)
  * @method void fillToBorder(int $x, int $y, ImageColor $borderColor, ImageColor $color)
  * @method void filter(int $filter, ...$args)
  * @method void flip(int $mode)
- * @method array ftText(float $size, float $angle, int $x, int $y, ImageColor $color, string $fontFile, string $text, array $options = [])
+ * @method array<int, int> ftText(float $size, float $angle, int $x, int $y, ImageColor $color, string $fontFile, string $text, array<string, mixed> $options = [])
  * @method void gammaCorrect(float $inputgamma, float $outputgamma)
- * @method array getClip()
+ * @method array{int, int, int, int} getClip()
  * @method int getInterpolation()
  * @method int interlace(?bool $enable = null)
  * @method bool isTrueColor()
  * @method void layerEffect(int $effect)
  * @method void line(int $x1, int $y1, int $x2, int $y2, ImageColor $color)
- * @method void openPolygon(array $points, ImageColor $color)
+ * @method void openPolygon(array<int, int> $points, ImageColor $color)
  * @method void paletteCopy(Image $source)
  * @method void paletteToTrueColor()
- * @method void polygon(array $points, ImageColor $color)
+ * @method void polygon(array<int, int> $points, ImageColor $color)
  * @method void rectangle(int $x1, int $y1, int $x2, int $y2, ImageColor $color)
  * @method mixed resolution(?int $resolutionX = null, ?int $resolutionY = null)
  * @method Image rotate(float $angle, ImageColor $backgroundColor)
  * @method void saveAlpha(bool $enable)
- * @method Image scale(int $newWidth, int $newHeight = -1, int $mode = IMG_BILINEAR_FIXED)
+ * @method Image scale(int $newWidth, int $newHeight = -1, int $mode = 3)
  * @method void setBrush(Image $brush)
  * @method void setClip(int $x1, int $y1, int $x2, int $y2)
- * @method void setInterpolation(int $method = IMG_BILINEAR_FIXED)
+ * @method void setInterpolation(int $method = 3)
  * @method void setPixel(int $x, int $y, ImageColor $color)
- * @method void setStyle(array $style)
+ * @method void setStyle(array<int, int> $style)
  * @method void setThickness(int $thickness)
  * @method void setTile(Image $tile)
  * @method void trueColorToPalette(bool $dither, int $ncolors)
- * @method array ttfText(float $size, float $angle, int $x, int $y, ImageColor $color, string $fontfile, string $text, array $options = [])
+ * @method array<int, int> ttfText(float $size, float $angle, int $x, int $y, ImageColor $color, string $fontfile, string $text, array<string, mixed> $options = [])
  * @property-read positive-int $width
  * @property-read positive-int $height
  * @property-read \GdImage $imageResource
@@ -144,6 +146,7 @@ class Image
 	/**
 	 * Returns RGB color (0..255) and transparency (0..127).
 	 * @deprecated use ImageColor::rgb()
+	 * @return array{red: int, green: int, blue: int, alpha: int}
 	 */
 	public static function rgb(int $red, int $green, int $blue, int $transparency = 0): array
 	{
@@ -211,6 +214,7 @@ class Image
 	 * Creates a new true color image of the given dimensions. The default color is black.
 	 * @param  positive-int  $width
 	 * @param  positive-int  $height
+	 * @param  ImageColor|array{red: int, green: int, blue: int, alpha?: int}|null  $color
 	 * @throws Nette\NotSupportedException if gd extension is not loaded
 	 */
 	public static function fromBlank(int $width, int $height, ImageColor|array|null $color = null): static
@@ -222,9 +226,9 @@ class Image
 
 		$image = new static(imagecreatetruecolor($width, $height));
 		if ($color) {
-			$image->alphablending(false);
-			$image->filledrectangle(0, 0, $width - 1, $height - 1, $color);
-			$image->alphablending(true);
+			$image->alphaBlending(false);
+			$image->filledRectangle(0, 0, $width - 1, $height - 1, $color);
+			$image->alphaBlending(true);
 		}
 
 		return $image;
@@ -233,23 +237,27 @@ class Image
 
 	/**
 	 * Returns the type of image from file.
-	 * @return ImageType::*|null
+	 * @param-out ?int  $width
+	 * @param-out ?int  $height
+	 * @return ?ImageType::*
 	 */
-	public static function detectTypeFromFile(string $file, &$width = null, &$height = null): ?int
+	public static function detectTypeFromFile(string $file, mixed &$width = null, mixed &$height = null): ?int
 	{
-		[$width, $height, $type] = @getimagesize($file); // @ - files smaller than 12 bytes causes read error
-		return isset(self::Formats[$type]) ? $type : null;
+		[$width, $height, $type] = Helpers::falseToNull(@getimagesize($file)); // @ - files smaller than 12 bytes causes read error
+		return $type && isset(self::Formats[$type]) ? $type : null;
 	}
 
 
 	/**
 	 * Returns the type of image from string.
-	 * @return ImageType::*|null
+	 * @param-out ?int  $width
+	 * @param-out ?int  $height
+	 * @return ?ImageType::*
 	 */
-	public static function detectTypeFromString(string $s, &$width = null, &$height = null): ?int
+	public static function detectTypeFromString(string $s, mixed &$width = null, mixed &$height = null): ?int
 	{
-		[$width, $height, $type] = @getimagesizefromstring($s); // @ - strings smaller than 12 bytes causes read error
-		return isset(self::Formats[$type]) ? $type : null;
+		[$width, $height, $type] = Helpers::falseToNull(@getimagesizefromstring($s)); // @ - strings smaller than 12 bytes causes read error
+		return $type && isset(self::Formats[$type]) ? $type : null;
 	}
 
 
@@ -416,7 +424,10 @@ class Image
 
 	/**
 	 * Calculates dimensions of resized image. Width and height accept pixels or percent.
+	 * @param  int|string|null  $newWidth
+	 * @param  int|string|null  $newHeight
 	 * @param  int-mask-of<self::OrSmaller|self::OrBigger|self::Stretch|self::Cover|self::ShrinkOnly>  $mode
+	 * @return array{int, int}
 	 */
 	public static function calculateSize(
 		int $srcWidth,
@@ -504,6 +515,7 @@ class Image
 
 	/**
 	 * Calculates dimensions of cutout in image. Arguments accepts pixels or percent.
+	 * @return array{int, int, int, int}
 	 */
 	public static function calculateCutout(
 		int $srcWidth,
@@ -624,6 +636,8 @@ class Image
 
 	/**
 	 * Calculates the bounding box for a TrueType text. Returns keys left, top, width and height.
+	 * @param  array<string, mixed>  $options
+	 * @return array{left: int, top: int, width: int, height: int}
 	 */
 	public static function calculateTextBox(
 		string $text,
@@ -668,7 +682,7 @@ class Image
 
 	/**
 	 * Saves image to the file. Quality is in the range 0..100 for JPEG (default 85), WEBP (default 80) and AVIF (default 30) and 0..9 for PNG (default 9).
-	 * @param  ImageType::*|null  $type
+	 * @param  ?ImageType::*  $type
 	 * @throws ImageException
 	 */
 	public function save(string $file, ?int $quality = null, ?int $type = null): void
@@ -744,6 +758,7 @@ class Image
 
 	/**
 	 * Call to undefined method.
+	 * @param  array<mixed>  $args
 	 * @throws Nette\MemberAccessException
 	 */
 	public function __call(string $name, array $args): mixed
@@ -771,7 +786,7 @@ class Image
 
 	public function __clone()
 	{
-		ob_start(function () {});
+		ob_start(fn() => '');
 		imagepng($this->image, null, 0);
 		$this->setImageResource(imagecreatefromstring(ob_get_clean()));
 	}
@@ -794,15 +809,18 @@ class Image
 	/**
 	 * Prevents serialization.
 	 */
-	public function __sleep(): array
+	public function __serialize(): array
 	{
 		throw new Nette\NotSupportedException('You cannot serialize or unserialize ' . self::class . ' instances.');
 	}
 
 
+	/**
+	 * @param  ImageColor|array{red: int, green: int, blue: int, alpha?: int}  $color
+	 */
 	public function resolveColor(ImageColor|array $color): int
 	{
-		$color = $color instanceof ImageColor ? $color->toRGBA() : array_values($color);
+		$color = $color instanceof ImageColor ? $color->toRGBA() : array_values($color + ['alpha' => 0]);
 		return imagecolorallocatealpha($this->image, ...$color) ?: imagecolorresolvealpha($this->image, ...$color);
 	}
 
